local AceOO = AceLibrary("AceOO-2.0");
local BS = AceLibrary("Babble-Spell-2.2");
if (not Yata.Class) then Yata.Class = {} end;
Yata.Class.TotemBar = AceOO.Class("AceEvent-2.0");
Yata.Class.TotemBar.virtual = true;
Yata.Class.ITotemBar = AceOO.Interface{CreateBar = "function", Reset = "function"};
Yata.Class.FlexTotemBar = AceOO.Class(Yata.Class.TotemBar, Yata.Class.ITotemBar);

------------------------
-- Bar template stuff --
------------------------
function Yata.Class.TotemBar.prototype:LoadConfig()
	local x = 0;
	local y = 0;
	local scale = 1;
	if(self.db.profile.Position) then
		x = self.db.profile.Position.x;
		y = self.db.profile.Position.y;
		scale = self.Header.Frame:GetEffectiveScale();
	else
		self.Header.Frame:ClearAllPoints();
		self.Header.Frame:SetPoint( "CENTER", WorldFrame, "CENTER", 0,0 );
		scale = self.Header.Frame:GetEffectiveScale();
		x = self.Header.Frame:GetLeft()*scale;
		y = self.Header.Frame:GetBottom()*scale;
	end
	local anchorTo;
	self.Header.Frame:ClearAllPoints();
	self.Header.Frame:SetPoint("BOTTOMLEFT", UIParent,"BOTTOMLEFT", x/scale, y/scale);
	self.Header.Frame:SetScale(self.db.profile.ButtonScale);
	for k,b in pairs(self.Buttons) do
		b.Frame:ClearAllPoints();
		if(b.ButtonID == 1) then
		    b.Frame:SetPoint("BOTTOMLEFT", self.Header.Frame,"BOTTOMLEFT", 20, 0);
		elseif(b.ButtonID < 5) then
		    b.Frame:SetPoint("BOTTOMLEFT", anchorTo, "BOTTOMRIGHT", 5,0);
		else
		    if(math.fmod(b.ButtonID,10) == 1) then
		    	anchorTo = self.Buttons[math.floor(b.ButtonID/10)].Frame;
		    	if(self.db.profile.InvertBar) then
		    		b.Frame:SetPoint( "TOP", anchorTo, "BOTTOM", 0, -5);
		    	else
		    		b.Frame:SetPoint( "BOTTOM", anchorTo, "TOP", 0, 5);
		    	end
		    else
			anchorTo = self.Buttons[b.ButtonID-1].Frame;
			if(self.db.profile.InvertBar) then
				b.Frame:SetPoint( "TOPLEFT", anchorTo, "BOTTOMLEFT", 0, -5);
			else
				b.Frame:SetPoint( "BOTTOMLEFT", anchorTo, "TOPLEFT", 0, 5);
			end
		    end
		    b.Frame:SetScale(self.db.profile.PopupButtonScale);
		end
		anchorTo = b.Frame;
	end
	self:HeaderMovement();
	self:SaveConfig();
	table.sort(self.Buttons,function(a,b) return a.ButtonID < b.ButtonID end);
	Yata:EditMacro();
end

function Yata.Class.TotemBar.prototype:SaveConfig()
	if not self.db.profile.Position then 
		self.db.profile.Position = {};
	end
	local scale = self.Header.Frame:GetEffectiveScale();
	scale = scale / self.db.profile.ButtonScale;
	local x = self.Header.Frame:GetLeft()*scale;
	local y = self.Header.Frame:GetBottom()*scale; 
	self.db.profile.Position.x = x;
	self.db.profile.Position.y = y;
end

function Yata.Class.TotemBar.prototype:Reset()
	if(not InCombatLockdown()) then
		self.db.profile.Position = nil;
		self:LoadConfig();
	end
end


function Yata.Class.TotemBar.prototype:Hide()
	self.Header.Frame:Hide();
end
function Yata.Class.TotemBar.prototype:Show()
	self.Header.Frame:Show();
end

function Yata.Class.TotemBar.prototype:init(db)
	Yata.Class.TotemBar.super.prototype.init(self);
	self.db = db;
	if(not self.db.profile.TotemSets) then self.db.profile.TotemSets = {} end;
	if(not self.db.profile.ActiveSet) then self.db.profile.ActiveSet = ""; end;
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN", "BarUpdateHandler");
	local totem = Yata.TotemData[BS["Strength of Earth Totem"]];
	self.CooldownID = totem[table.getn(totem)].SpellID;
end

-----------------
-- Keybindings --
-----------------

function Yata.Class.TotemBar.prototype:SetupKeybindings()
	for k,b in pairs(self.Buttons) do
		self:CreateTotemKeybindingOption(b.ButtonID);
		local hotkey = getglobal( b.Frame:GetName() .. "HotKey" );
		hotkey:SetText(GetBindingText(self.db.profile.Keybindings[b.ButtonID], "KEY_"));
	end
end

-- Binding functions basicly from ClosetGnome by rabbit
function Yata.Class.TotemBar.prototype:ClearBinding(ID)
	if not button then return end
	local oldbind = self.db.profile.Keybindings[ID];
	if not oldbind then return end
	self.db.profile.Keybindings[ID] = nil;
	SetBinding(oldbind);
	local hotkey = getglobal( self.Buttons[ID].Frame:GetName() .. "HotKey" );
	hotkey:SetText("");
end

function Yata.Class.TotemBar.prototype:RegisterBinding(key, ID)
	if not key or not button then return end
	if not AceLibrary("AceConsole-2.0").keybindingValidateFunc(key) then
		self:Print(string.format("%s is not a valid keybinding.", "|cffd9d919"..key.."|r"))
		return
	end
	for k, v in pairs(self.db.profile.Keybindings) do
		if v == key and k ~= ID then
			self.db.profile.Keybindings[k] = nil;
		elseif k == ID and v ~= key then
			self.db.profile.Keybindings[k] = nil;
			SetBinding(v) -- remove old binding
		end
	end
	self.db.profile.Keybindings[ID] = key;
	SetBindingClick(key, self.Buttons[ID].Frame:GetName());
	local hotkey = getglobal( self.Buttons[ID].Frame:GetName() .. "HotKey" );
	hotkey:SetText(GetBindingText(key, "KEY_"));
	-- save our new bindings
	SaveBindings(GetCurrentBindingSet());
end
-- basically from Bartender3
function Yata.Class.TotemBar.prototype:CreateTotemKeybindingOption(ID)
	local options = Yata.options.args.keybindings.args;
	local text = ""; 
	local shorttext = "";
	if(ID < 5) then
		text = self.Buttons[ID].Totem.Element.." Main Button ";
		shorttext = self.Buttons[ID].Totem.Element.."MainButton";
	else
		text = self.Buttons[ID].Totem.Element.." Menu Button "..math.fmod(ID,10);
		shorttext = self.Buttons[ID].Totem.Element.."MenuButton"..math.fmod(ID,10);
	end
	if(not self.db.profile.Keybindings) then
		self.db.profile.Keybindings = {};
	end
	options[shorttext] = {
		type = "text",
		name = text,
		desc = "Assign a Keybinding to",
		validate = "keybinding",
		order = ID,
		get = function()
			return self.db.profile.Keybindings[ID];
		end,
		set = function(k)
			if k == nil then
				self:ClearBinding(ID);
			else
				self:RegisterBinding(k,ID);
			end
		end,
	}
end


----------------
-- Totem Sets --
----------------

function Yata.Class.TotemBar.prototype:AddTotemSet(name)
	if(name == "") then return end;
		local temp = {};
		local oldone = false;
		Yata:Print(name);
		if(self.db.profile.TotemSets[name]) then 
			oldone = true;
		end
		if(self.db.profile.ActiveSet ~= "") then
			for k,v in pairs(self.db.profile.TotemSets[self.db.profile.ActiveSet]) do 
				table.insert(temp,v);
			end
		end
		for k,v in pairs(self.TempTotemSet) do
			table.insert(temp,v);
		end
		self.db.profile.TotemSets[name] = temp;
		-- clean up code incoming --
		self:CleanupSet(name);
		self.db.profile.ActiveSet = name;
		Yata.options.args.totemsets.args.header.name = "Active Set: "..name;
		if(not oldone) then
			self:CreateTotemSetOption(name);
		end
		Yata:UpdateText();
		
end
function Yata.Class.TotemBar.prototype:CleanupSet(name)
	-- not yet in --
end

function Yata.Class.TotemBar.prototype:DeleteTotemSetWrapper(name)
	self.ToDelete = name;
	StaticPopup_Show("YataDeleteSet",name);
end

function Yata.Class.TotemBar.prototype:DeleteTotemSet()
	local name = self.ToDelete;
	if(name == self.db.profile.ActiveSet) then
		Yata:Print("Please reset first, before you delete an active set");
	else
		self.db.profile.TotemSets[name] =  nil;
		Yata:Print("Deleted "..name);
		-- update options now --
		Yata.options.args.totemsets.args.loadset.args[name] = nil;
		Yata.options.args.totemsets.args.deleteset.args[name] = nil;
	end
end

function Yata.Class.TotemBar.prototype:ResetSet()
	self:UnloadTotemSet();
	self.db.profile.ActiveSet = "";
	Yata.options.args.totemsets.args.header.name = "No active Set";
	Yata:UpdateText();
	Yata:EditMacro();
end

function Yata.Class.TotemBar.prototype:LoadTotemSet(name)
	if(name == "") then return end;
	if(name ~= self.db.profile.ActiveSet) then
		self:UnloadTotemSet();
	end
	if(not self.db.profile.TotemSets[name]) then 
		Yata:Print("Set: "..name.." does not exist");
		self.db.profile.ActiveSet = "";
		return 
	end;
	for k,v in pairs(self.db.profile.TotemSets[name]) do
		if(#v ~= 2) then 
			Yata:Print("Problem with Set, ignoring current set, you should delete it");
			return
		end
		local s1totem = v[1];
		local s2totem = v[2];
		self:Permutate(s1totem,s2totem);
	end
	self.db.profile.ActiveSet = name;
	Yata.options.args.totemsets.args.header.name = "Active Set: "..name;
	Yata:UpdateText();
	Yata:EditMacro();
end

function Yata.Class.TotemBar.prototype:Permutate(s1totem,s2totem)
	local start = 0;
	local stop = 0;
	for l,w in pairs(self.Buttons) do
		if(w.Totem.Name == s1totem) then
			start = l;
		elseif(w.Totem.Name == s2totem) then
			stop = l;
		end
		if(start > 0 and stop > 0) then break; end
	end
	if(start == 0 or stop == 0) then
		Yata:Print("Problem with Set, ignoring current value, you should delete it");
		return
	end
	local s1 = self.Buttons[start].Totem;
	local s2 = self.Buttons[stop].Totem;
	self.Buttons[start]:Assign(s2);
	self.Buttons[stop]:Assign(s1);
end

function Yata.Class.TotemBar.prototype:UnloadTotemSet()
	if(self.db.profile.ActiveSet == "") then return end;
	-- unloading code here --
	-- basically permutating the whole thing backwards --
	local set = self.db.profile.TotemSets[self.db.profile.ActiveSet];
	for i=#set, 1, -1 do
			local s1totem = (set[i])[1];
			local s2totem = (set[i])[2];
			self:Permutate(s2totem,s1totem);
	end
	self.db.profile.ActiveSet = "";
end

function Yata.Class.TotemBar.prototype:SetupTotemSets()
	for k,v in pairs(self.db.profile.TotemSets) do
		self:CreateTotemSetOption(k);
	end
end

function Yata.Class.TotemBar.prototype:CreateTotemSetOption(name)
	local options = Yata.options.args.totemsets.args.loadset.args;
	options[name] = {
		name = name,
		desc = "Load "..name,
		type = "execute",
		func = function() self:LoadTotemSet(name) end,
	}
	options = Yata.options.args.totemsets.args.deleteset.args;
	options[name] = {
		name = name,
		desc = "Delete "..name,
		type = "execute",
		func = function() 
			self:DeleteTotemSetWrapper(name);
			end,
	}
end
-----------------
-- Swap Button --
-----------------

function Yata.Class.TotemBar.prototype:ButtonDrag(start, ID)
	if(self.db.profile.ButtonLocked) then return end;
	if(start) then
		self.StartSwapButton = ID;
		PickupSpell( self.Buttons[ID].Totem.SpellID, BOOKTYPE_SPELL );
	else 
		self.StopSwapButton = ID;
		ClearCursor();
		self:SwapButton(self.StartSwapButton, self.StopSwapButton);
		self.StartSwapButton = nil;
		self.StopSwapButton = nil;
	end
end
function Yata.Class.TotemBar.prototype:SwapButton(start, stop)
	-- Swapping buttons with the same element is easy
	local s1 = self.Buttons[start].Totem;
	local s2 = self.Buttons[stop].Totem;
	local s1ele = s1.Element;
	local s2ele = s2.Element;
	if(s1ele == s2ele) then
		if(self.EditSet) then
			local temp = {s1.Name,s2.Name};
			table.insert(self.TempTotemSet,temp);
		else
			s1pos = Yata:GetPosByTotem(s1);
			s2pos = Yata:GetPosByTotem(s2);
			(self.db.profile.TotemsOrder[s1ele])[s2pos] = s1.Name;
			(self.db.profile.TotemsOrder[s1ele])[s1pos] = s2.Name;
		end
		-- Same Element
			self.Buttons[start]:Assign(s2);
			self.Buttons[stop]:Assign(s1);
	-- The more complicated part aka swapping whole elements is not yet implemented 
	elseif(s1ele ~= s2ele and start < 5 and stop < 5) then
		self:Print("Please edit the savedvariables for now");
	end
end

---------------------
-- Header Movement --
---------------------

function Yata.Class.TotemBar.prototype:HeaderMovement()
	local b = self.Header;
	b.Frame:EnableMouse(true);
	b.Frame:SetMovable(true);
	b.Frame:SetScript("OnDragStart", function()
		if(not self.db.profile.Locked) then
			this:StartMoving();
		end
	end)
	b.Frame:SetScript("OnDragStop", function()
		this:StopMovingOrSizing();
		self:SaveConfig();
	end);
end
-------------------
-- Flex Totembar --
-------------------

function Yata.Class.FlexTotemBar.prototype:init(db)
	Yata.Class.FlexTotemBar.super.prototype.init(self,db);
	-- flexbar specific below --
	hooksecurefunc("SecureStateAnchor_RunChild",Yata.Class.ControlButton.prototype.ControlOnEnter);
end

-- Buttons: 1-4 are the Control Buttons, 11-1X are the MouseOver Buttons for Element 1, 21-2X for Element 2 etc.
function Yata.Class.FlexTotemBar.prototype:CreateBar()
	self.Buttons = {};			
	-- First we create the Header
	self.Header = Yata.Class.Header:new();
	-- After the header, we generate 4 Control buttons
	for i = 1,4 do 
		self.Buttons[i] = Yata.Class.ControlButton:new(self.db.profile.TotemsOrder[i].."CONTROL",i,self.Header);
	end
	-- Then we populate them and create other buttons on the fly
	-- or we load config data
	for k,v in ipairs(Yata.TotemData) do
		local totem = Yata.TotemData[v];
		totem = totem[table.getn(totem)];
		local number = 0;
		local pos = Yata:GetPosByTotem(totem);
		if(pos == 1) then
			number = pos * Yata:ReturnOrderByElement(totem.Element);
		elseif(pos > 1) then
			number = pos - 1 + Yata:ReturnOrderByElement(totem.Element)*10;
			-- case if there is a new totem in the middle 
			if(self.Buttons[number]) then
				number = number + 1;
			end
		elseif(pos == 0) then
			pos = Yata:ReturnOrderByElement(totem.Element);
			if(not self.Buttons[pos].Totem) then
				number = pos;
			else
				number = pos*10 + Yata.ElementsCount[totem.Element];
			end
		end
		if(not self.Buttons[number]) then
			self.Buttons[number] = Yata.Class.StandardButton:new(totem.Element,number,self.Header);
		end
		self.Buttons[number]:Assign(totem);
		Yata.ElementsCount[totem.Element] = Yata.ElementsCount[totem.Element] + 1;
	end
	-- save code --
	local count = 0;
	for i=1,4 do
		self.db.profile.TotemsOrder[self.db.profile.TotemsOrder[i]] = {};
	end
	for k,v in pairs(self.Buttons) do
		if(v.ButtonID < 5) then
			(self.db.profile.TotemsOrder[v.Totem.Element])[1] = v.Totem.Name;
			count = count + 1;
		else 
			(self.db.profile.TotemsOrder[v.Totem.Element])[math.fmod(v.ButtonID,10)+1] = v.Totem.Name;
			count = count + 1;
		end	
	end
	self.db.profile.TotemsCount = count;
	-- load set here --
	self:LoadTotemSet(self.db.profile.ActiveSet);
	self:SetupTotemSets();
	self:SetupKeybindings();
end

function Yata.Class.FlexTotemBar.prototype:BarUpdateHandler()
	self:UpdateCooldown();
end
function Yata.Class.FlexTotemBar.prototype:UpdateCooldown()
	-- fugly hack here but does the trick, we simply always use the SpellID of "Strength of Earth Totem"
	-- to get around cooldown
	local start, duration, enable = GetSpellCooldown(self.CooldownID, BOOKTYPE_SPELL );
	for i=1,4 do
		local cooldown = getglobal( self.Buttons[i].Frame:GetName() .. "Cooldown" );
		CooldownFrame_SetTimer( cooldown, start, duration, enable );
	end
end


-----------------
-- Button Swap --
-----------------

function Yata.Class.FlexTotemBar.prototype:SetupSwap(locked)
	if(not locked) then
		for k,b in pairs(self.Buttons) do
			b.Frame:SetAttribute("showstates", "*");
			b.Frame:Show();
		end
	else 
		for k,b in pairs(self.Buttons) do
			if(b.ButtonID > 10) then
				b.Frame:SetAttribute("showstates", math.floor(b.ButtonID/10));
				b.Frame:Hide();
			end
		end
	end

end